﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Telerik.Charting;
using Telerik.WinControls.UI;

namespace _1075245_RadChartViewZoomInRange
{
    public partial class Form2 : Form
    {
        double lineXCorrection, barXCorrection, stepLineXCorrection;

        public Form2()
        {
            InitializeComponent();

            this.SetupScatterChart();
            this.SetupLineChart();
            this.SetupBarChart();
            this.SetupSteplLineChart();

            this.radChartView2.ShowPanZoom = true;
            this.radChartView3.ShowPanZoom = true;
            this.radChartView4.ShowPanZoom = true;

            ChartPanZoomController panZoomController = new ChartPanZoomController();
            panZoomController.PanZoomMode = ChartPanZoomMode.Both;
            this.radChartView1.Controllers.Add(panZoomController);

            this.radChartView1.View.ZoomChanged += View_ZoomChanged;
            this.radChartView1.View.PanChanged += View_PanChanged;

            double baseCount = this.radChartView1.Series[0].DataPoints.Count;
            lineXCorrection = this.radChartView2.Series[0].DataPoints.Count / baseCount;
            barXCorrection = this.radChartView3.Series[0].DataPoints.Count / baseCount;
            stepLineXCorrection = this.radChartView4.Series[0].DataPoints.Count / baseCount;
        }

        private void View_PanChanged(object sender, EventArgs e)
        {
            IChartView view = ((IChartView)this.radChartView1.View);
            double panX = view.PlotOriginX;
            double panY = view.PlotOriginY;

            this.radChartView2.Pan(panX * lineXCorrection, panY);
            this.radChartView3.Pan(panX * barXCorrection, panY);
            this.radChartView4.Pan(panX * stepLineXCorrection, panY);
        }

        private void View_ZoomChanged(object sender, EventArgs e)
        {
            IChartView view = ((IChartView)this.radChartView1.View);
            double zoomX = view.ZoomWidth;
            double zoomY = view.ZoomHeight;

            this.radChartView2.Zoom(zoomX * lineXCorrection, zoomY);
            this.radChartView3.Zoom(zoomX * barXCorrection, zoomY);
            this.radChartView4.Zoom(zoomX * stepLineXCorrection, zoomY);
        }

        private void SetupScatterChart()
        {
            ScatterSeries scatterSeries = new ScatterSeries();
            Random r = new Random();
            for (int i = 0; i < 25; i++)
            {
                scatterSeries.DataPoints.Add(new ScatterDataPoint(i, r.Next(100)));
            }

            this.radChartView1.Series.Add(scatterSeries);

            ((LinearAxis)this.radChartView1.Axes[0]).LabelFitMode = AxisLabelFitMode.None;
            ((LinearAxis)this.radChartView1.Axes[0]).LabelInterval = 1;
            ((LinearAxis)this.radChartView1.Axes[0]).MajorStep = 1;
        }

        private void SetupLineChart()
        {
            LineSeries lineSeries = new LineSeries();
            Random r = new Random();
            for (int i = 0; i < 35; i++)
            {
                lineSeries.DataPoints.Add(new CategoricalDataPoint(r.Next(100), i));
            }

            this.radChartView2.Series.Add(lineSeries);
        }

        private void SetupBarChart()
        {
            BarSeries barSeries = new BarSeries();
            Random r = new Random();
            for (int i = 0; i < 45; i++)
            {
                barSeries.DataPoints.Add(new CategoricalDataPoint(r.Next(100), i));
            }

            this.radChartView3.Series.Add(barSeries);
        }

        private void SetupSteplLineChart()
        {
            SteplineSeries stepLineSeries = new SteplineSeries();
            Random r = new Random();
            for (int i = 0; i < 55; i++)
            {
                stepLineSeries.DataPoints.Add(new CategoricalDataPoint(r.Next(100), i));
            }

            this.radChartView4.Series.Add(stepLineSeries);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            IChartView view = ((IChartView)this.radChartView1.View);

            foreach (var series in this.radChartView1.Series)
            {
                foreach (var dp in series.DataPoints)
                {
                    ScatterDataPoint sdp = (ScatterDataPoint)dp;
                    if (CheckIfDataPointIsVisible(view, sdp))
                    {
                        Console.WriteLine(sdp.XValue + " " + sdp.YValue);
                    }
                }
            }
        }

        private bool CheckIfDataPointIsVisible(IChartView view, ScatterDataPoint sdp)
        {
            bool inXRange = sdp.LayoutSlot.X >= -(view.PlotOriginX) && sdp.LayoutSlot.X <= -view.PlotOriginX + view.ViewportWidth;
            bool inYRange = sdp.LayoutSlot.Y >= -(view.PlotOriginY) && sdp.LayoutSlot.Y <= -view.PlotOriginY + view.ViewportHeight;

            return inXRange && inYRange;
        }
    }
}
